@@ -1,6 +1,3 @@
-from typing import Type
-from typing import Union
-
 from mindflow.db.objects.static_definition.model import ModelID
 from mindflow.db.objects.static_definition.model_type import ModelType
 from mindflow.utils.enum import ExtendedEnum
